import template from '@babel/template';

const templateCjsDefault = `module.exports = function (%%args%%) { %%body%% }`;
const templateCjsNamed = `exports.%%name%% = function (%%args%%) { %%body%% }`;
const templateEsmDefault = `export default function (%%args%%) { %%body%% }`;
const templateEsmNamed = `export function %%name%% (%%args%%) { %%body%% }`;
const templateGlobalDefault = `%%body%%`;
const templateGlobalNamed = `%%name%% = function (%%args%%) { %%body%% }`;
const templateIifeDefault = `(function (%%args%%) { %%body%% })(%%args%%)`;
const templateIifeNamed = `%%name%% = function (%%args%%) { %%body%% }(%%args%%)`;

function babelPluginTransformGlobals(api, opts) {
  const {
    types: t
  } = api;
  const run = []; // options

  const args = [].concat(opts.args || []).map(arg => t.identifier(arg));
  const name = 'name' in opts ? String(opts.name) : 'default';
  const format = ['cjs', 'esm', 'iife', 'global'].find(format => format === opts.format);
  const buildTemplate = template(name === 'default' ? format === 'cjs' ? templateCjsDefault : format === 'iife' ? templateIifeDefault : format === 'global' ? templateGlobalDefault : templateEsmDefault : format === 'cjs' ? templateCjsNamed : format === 'iife' ? templateIifeNamed : format === 'global' ? templateGlobalNamed : templateEsmNamed);

  if (name === 'default' && format === 'global') {
    throw Error('The "global" option requires a "name"');
  }

  return {
    name: 'func-wrap',
    visitor: {
      Program: {
        exit(path) {
          if (!run.includes(this)) {
            run.push(this);
            const {
              body
            } = path.node;
            const templateOptions = name === 'default' ? {
              body,
              args
            } : {
              name,
              body,
              args
            };
            const ast = buildTemplate(templateOptions);
            path.replaceWith(t.program([ast]));
          }
        }

      }
    }
  };
}

export default babelPluginTransformGlobals;
//# sourceMappingURL=index.mjs.map
